/*
 * Copyright (c) 2005 INdT.
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __XMPP_PRIVACY_LIST_H__
#define __XMPP_PRIVACY_LIST_H__

#include <talk/base/logging.h>
#include <talk/p2p/base/helpers.h>
#include <talk/base/criticalsection.h>
#include <talk/base/sigslot.h>
#include <talk/xmpp/xmppengine.h>
#include <talk/xmpp/xmpptask.h>
#include <talk/xmpp/jid.h>
#include <talk/xmpp/constants.h>

#include <map>
#include <string>

#include "xmpp_privacy_c.h"

namespace buzz {
    class XmlElement;
}

class XmppPrivacyList : public buzz::XmppTask {
public:
    XmppPrivacyList(buzz::Task *parent);
    ~XmppPrivacyList();
        
    void RequestLists();
    void RequestRules(const std::string &name);
    std::string GetDefault()
    {   return default_list_; }
    void SetDefault(const std::string &name);
    std::string GetActive()
    {   return active_list_; }
    void SetActive(const std::string &name);
    void SetRuleRetrievedCb(xmpp_privacy_rule_retrieved_cb_type cb, void *data)
    {   rule_cb_ = cb; rule_data_ = data; }
    void SetListsRetrievedCb(xmpp_privacy_lists_retrieved_cb_type cb, void *data)
    {   lists_cb_ = cb; lists_data_ = data; }
    void AddRule(const std::string &name, const std::string &type,
        const std::string &value, const std::string &action);
    void RemoveRule(const std::string &name, const std::string &type,
        const std::string &value, const std::string &action);

    int ProcessStart();

private:
    bool HandleStanza(const buzz::XmlElement *stanza);
    buzz::XmlElement *GenerateXml(buzz::XmlElement *list = NULL, const std::string &type = buzz::STR_SET);
    std::string GenerateId()
    {
        std::string id_str;
        std::stringstream st;
        st << cricket::CreateRandomId();
        st >> id_str;
        return id_str;
    }
        
    buzz::XmlElement *active_list_rules_;
    std::string active_list_;
    std::string default_list_;
    cricket::CriticalSection crit_;
    xmpp_privacy_rule_retrieved_cb_type rule_cb_;
    void *rule_data_;
    xmpp_privacy_lists_retrieved_cb_type lists_cb_;
    void *lists_data_;    
};

#endif 
